/*************************************************************************
 FILE:				S_CONFIG.H     
 CREATION DATE:		10 May 1996
 LAST MODIFIED:		24 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file defines the session_config class, which is the 
 principal conceptual entity in WAPWAD's conceptual model. session_config
 defines the configuration for experimental block sessions, practice sessions, 
 and experimentor sessions -- hence the name session_config. session_config 
 encapsulates several definitions and user preferences, including:
  
 (1) task--that is, what participants need to do to complete a trial; 
 (2) error handling technique--that is,
 	 how to deal with trials that were not successfully completed;  
 (3) block--a range of target diameters, target
 	 distances, and target angles to be crossed;
 (4) practice session length--the experimentor can define a practice
 	 or warm-up session bounded by either time or number of trials;
 (5) home square placement--whether the home square is always placed in 
 	 the center of the screen at the beginning of each trial, or whether
 	 it should be placed at a random location; and
 (6) data to collect--should only performance data (i.e., trial time and
 	 errors) be collected, or should we collect movement microstructure 
 	 data as well? 
 	    
 I see the session_config as providing bare-bones support for experiment
 definition.  While it does not support the tracking of participants through
 experiments, it does enable experimentors to define basic parameters for an
 experiment, and to save those parameters for repeated use throughout the
 duration of the experiment.  For the sake of simplicity, I have avoided 
 implementing a full-blown experiment manager; that remains an intriguing 
 possibility for future work.
 *************************************************************************/  

#ifndef __S_CONFIG_H
#define __S_CONFIG_H

#include <stdlib.h>
#include <strstrea.h>
//#include "boolean.h"
#include "t_data.h"
#include "list.h"
#include "pa_ses.h"
#include "pr_ses.h"
#include "ex_ses.h"
#include "dtd_db.h"
#define NUM_KEYWORDS 12

class session_config {    
	
	private:  
		// Note:  keyword 'errror' indicates that an _invalid_ keyword
		// was encountered.
		enum keyword {	task_type, target_diameters, target_distances,
						target_angles, trials_per_combination, number_of_blocks,
						error_handling_technique, practice_session_units,
						practice_session_length, home_square_loc, 
						data_to_collect, save_practice_data, error };
			
		int my_atoi(const char *str);
		//*************************************************************************
    	// GIVEN  : A character string.
    	// TASK   : Let's face it: C's atoi() function is lame, since it cannot
		//			distinguish between a valid integer 0 value and a non-integer 
		//			value; in both cases, atoi returns 0!  Aaargh.  By making the
		//			assumption that only non-negative integers should be acceptable,
		//			this function remedies that shortcoming.
    	// RETURN : The non-negative integer corresponding to the value read in, or
		//			a value less than 0 if the value could not be converted, or if
		//			the value really is less than 0.
    	//*************************************************************************

		double my_atof(const char *str);
		//  See comments for my_atoi () above.

		void absorb_comment(ifstream& inf);
		//*************************************************************************
    	// GIVEN  : An ifstream reference from which to read.
    	// TASK   : If possible, read characters from inf up to and 
    	//			including an endline.  If eof is encountered before an
    	//			an endline, merely return.
    	// RETURN : Nothing
    	//*************************************************************************
        
        BOOL eatwhite(ifstream &inf);
        //*************************************************************************
    	// GIVEN  : An ifstream reference from which to read.
    	// TASK   : Eat non-newline whitespace characters from inf up to 
     	//			either a non-whitespace char or a newline.
    	// RETURN : FALSE if a non-whitespace, non-newline char was read, and
    	//			TRUE if a newline char was read.
    	//*************************************************************************
      
        BOOL read_val(ifstream &inf, char *val);
        //*************************************************************************
    	// GIVEN  : An ifstream reference from which to read.
    	// TASK   : Read characters from inf up to a whitespace character.
    	// RETURN : FALSE if a whitespace character was not encountered after
    	//			having read 80 characters, TRUE otherwise.
    	//*************************************************************************    
        
        keyword read_kw_and_vals(ifstream& inf, int c_line, ostrstream& s_mes);
        //*************************************************************************
    	// GIVEN  : An ifstream reference from which to read, the number
    	//			of the line we're currently on, and a string stream to which
    	//			to write a status message.
    	// TASK   : If possible, read a keyword-values pair from inf, updating
    	//			this's data members appropriately, 
    	// RETURN : The keyword read in, if it could be read in successfully.
    	//			If an error is encountered, describe it as lucidly as 
    	//			possible in s_mess, and return error as the keyword.
    	//*************************************************************************
        
        void fill_in_missing_kw(keyword key);
		//*************************************************************************
    	// GIVEN  : A keyword key that was not encountered in the config. file.
    	// TASK   : Update the data member corresponding to key with 
    	//			appropriate default values.
    	// RETURN : Nothing
    	//*************************************************************************
	     
	    trial_data* get_trial_data(int dist, int diam, int ang);      
	    //*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Return a pointer to an instance of an appropriate trial_data
    	//			subclass, which can be obtained by accessing the appropriate
    	//			field of the class data member dtd.  The "appropriate" member 
    	//			of dtd can be ascertained by peeking at the value of class data
    	//			member d_to_collect.
    	// RETURN : A pointer to a fresh instance of trial_data.
    	//*************************************************************************
	     
	     
	public:
		/* DATA */
		// We support three error handling techniques:
		// redo-at_end:		If trial is not performed successfully, place it in an
		//					'error trial' queue; trials in the 'error trial' queue are
		//					the presented to the user after all trials have been presented.
		// dont_redo:		If trial is not performed successfully, do not force user to redo it.
		
		// We support two forms in which the duration of a practice session may
		// be specified: in terms of trials and minutes.
		enum hsquare_locs {center, random};
		// Where should the home square be located at the beginning of each
		// trial? 
		enum d_to_collect {performance, micro};
		// Should we collect performance data only, or movement 
		// microstructure data as well?
		list *labels_list;  // The valid task labels  
		dtd_database *dtd_db; // The database of data task definitions
		dtd_database::data_task_def *dtd; // The data_task_def struct 
		//associated with this session_config's task   
		list target_diams; // Diameter of target circles, in mm
		list target_dists; // Distances of target circles from start, in mm
		list target_angs;    // Angles of target circle w.r.t. home, in degrees
		int trials_per_comb; // Number of trials of each diam/distance/angle combination
		int num_blocks;		// Number of blocks, each consisting of target_diams*target_dists*target_angs*trials_per_comb trials
		int ps_len; // The length of the practice session, in either trials or minutes
		practice_session::psession_units ps_units; // Units in which practice session, if any, is specified
		participant_session::ehandling_techniques err_tech;	// Error handling technique
		hsquare_locs home;       
		d_to_collect collect_data;
		BOOL save_pdata; // Should practice data be saved too?
		                                                 
		/* METHODS */
		
		session_config(BOOL set_defaults, dtd_database* dtdd);
		//*************************************************************************
    	// GIVEN  : A BOOL 'set_defaults' and a reference to the 
    	//			data task definition database in which the various
    	//			td_generator and tp_generator instances can be looked up.
    	// TASK   : Task_block_def constructor:  If set_defaults is TRUE, then
    	//			initialize a new instance of session_config with 
    	//			reasonable default values. Otherwise, don't bother 
    	//			setting any of the data members; they will be set later
    	//			in the read_from_file method.
    	// RETURN : A fresh instance of session_config.
    	//*************************************************************************

		session_config (ifstream& infile, dtd_database* dtdd);
		//*************************************************************************
    	// GIVEN  : A file stream and a reference to a data task definition database.
    	// TASK   : Read in a session configuration from the stream.
    	// RETURN : A fresh instance of session_config.
    	//*************************************************************************

		char* read_from_file(ifstream& infile);
		//*************************************************************************
    	// GIVEN  : Reference to a session_config initialization file.
    	// TASK   : If possible, initialize values of session_config based on 
    	//			contents of infile.
    	// RETURN : A char* containing the result of the operation.  If the
    	//			instance could be properly initialized from the file, 
    	//			then return "SUCCESS". If there exists an
    	//			error in infile, return an appropriate error message
    	//			beginning with "ERROR". 
    	//			
    	//*************************************************************************
        
        BOOL write_to_file(ofstream& outfile);
        //*************************************************************************
    	// GIVEN  : Reference to a session_config output file.
    	// TASK   : Write instance to output file.
    	// RETURN : TRUE if the instance could be properly output to the 
    	//			file, FALSE otherwise.
    	//*************************************************************************
      
		participant_session* generate_participant_session(int seed, ofstream* out,
				ofstream* trailsOut, const CString& participantID, const CString& sessNum);
		//*************************************************************************
    	// GIVEN  : Basic parameters of a session.
    	// TASK   : Generate a single participant session (one block) based on the 
		//			configuration specified in self's data members.
    	// RETURN : A new session instance ready to be fed to a session_presenter
    	//			instance.
    	//*************************************************************************
		
		practice_session* generate_practice_session();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Generate a single participant practice session (bounded by 
    	//			specifed practice session length) based on the 
		//			configuration specified in self's data members.
    	// RETURN : A new session instance ready to be fed to a session_presenter
    	//			instance.
    	//*************************************************************************
		

		experimentor_session* generate_experimentor_session();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Generate an experimentor session (open-ended) based on the 
		//			configuration specified in self's data members.
    	// RETURN : A new session instance ready to be fed to a session_presenter
    	//			instance.
    	//*************************************************************************
	
		trial_presenter* get_tp();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Grab appropriate instance of trial_presenter. 
    	// RETURN : (An instance of tp_generator that can be used to produce an
		//  		appropriate instance of trial_presentor.  Since they 
		//			depend on Windows environment variables, trial_presentor instances 
		//			need to be instantiated at the top level of the program.)
    	//*************************************************************************

		~session_config();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Return free store space associated with this instance of 
		//			session_config.
    	// RETURN : Nothing
    	//*************************************************************************

	}; 

#endif
