/*************************************************************************
 FILE:				SESSION.H     
 CREATION DATE:		10 May 1996
 LAST MODIFIED:		19 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 Class session serves as an abstract base class	that encapsulates the 
 fundamental characteristics of a session in WAPWAD's conceputal model.
 A session has three main responsibilities:  (1) Holding session trials,
 and dispatching them to the main program on request; (2) taking all
 trials back after they have been presented to the user and logging them
 appropriately; and (3) writing logged session data to a file on request.
 Subclasses of session specialize the manner in which trials are 
 generated (e.g., a session subclass for generating participant block 
 sessions should generate a fixed number of trials, whereas a session
 subclass for generating experimentor trials should generate trials
 ad infinitum).
*************************************************************************/  

#ifndef __SESS_H
#define __SESS_H

#include "pqueue.h"  
#include "list.h"
#include "t_data.h"

class session_config;

class session {
	 
	public:     
		// DATA
		int total_trials;		// How many trials are available to be completed?
		int trials_completed;	// Ongoing count of trials completed without error
		int trials_presented;	// Ongoing count of trials presented to user
		int total_trial_time;	// Aggregate time of all completed trials
		int mean_trial_time;	// Dynamic computation of mean trial time
		bool session_stopped;	// Session stopped prematurely at user's request

		// Identification of this session
		ofstream* outFile;		// The file to store trial total data
		ofstream* trails_out;	// The file individual trial trails
		CString participantID;
		CString sessionNum;
		
	
		virtual trial_data* get_next_trial(); 
		//********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Dequeue next trial to be presented.
    	// RETURN : A pointer to the next trial to be presented.
    	//********************************************************************
	
		virtual void log_trial(trial_data *td) = 0; 
		//********************************************************************
    	// GIVEN  : A pointer to the trial just presented and completed. 
    	// TASK   : Register the trial as having been completed.
    	// RETURN : Nothing.
    	//********************************************************************
	
		virtual void write_out (); 
		//********************************************************************
    	// GIVEN  : Nothing. 
    	// TASK   : Open the fileName and write out this session.
    	// RETURN : Nothing.
    	//********************************************************************

		virtual void stop_session (); 
		//********************************************************************
    	// GIVEN  : Nothing. 
    	// TASK   : Record premature termination of this session.
    	// RETURN : Nothing.
    	//********************************************************************

		virtual bool more_blocks ();

		virtual void next_block ();

		virtual void start_first_block ();

		virtual int get_curr_block () { return curr_block + 1;};

		virtual int get_num_blocks () { return num_blocks;};

		friend ofstream& operator << (ofstream& outfile, session& s); 
		//********************************************************************
    	// GIVEN  : References to an output file and a session object. 
    	// TASK   : Write as much of the session as has been completed to the
    	//			outfile. 
    	// RETURN : The ofstream reference, as is customary. 
    	// WARNING: Once a trial_data instance has been written to the data
    	//		 	file, it is permanently destroyed.
    	//********************************************************************

		//session ();
		//********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Construct a sessoin.
    	// RETURN : A new session.
    	//********************************************************************

		~session();
		//********************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Return all heap-allocated memory associated with this.
    	// RETURN : Nothing.
    	//********************************************************************

	protected:
	 	// DATA
		list block;				// All the trials for one block
		pqueue pending_trials;	// The trials that haven't yet been presented.
		list completed_trials;	//The trials that have been presented.
		int curr_block;			// The block number within this sequence of blocks
		int num_blocks;			// The total blocks in this session
		
		// METHODS		
		session(int num_trials, int blocks, int seed, ofstream* out = NULL, ofstream* trailsOut = NULL,
			const CString& participantID = CString (""), const CString& sessNum = CString ("")); 
		//********************************************************************
    	// GIVEN  : The number of trials with which to initialize the,
    	//			session, the number of blocks in the session,
		//          along with the seed with which to initialize
    	//			the first-in-random-out priority queue and various file parameters.
    	// TASK   : Instantiate a new instance of session.
    	// RETURN : A new session instance.
    	// NOTE   : The constructor should be protected since only the friend class
		//			session_config should be allowed to instantiate sessions.
    	//********************************************************************
	 
		virtual void add_trial(trial_data *td); 
		//********************************************************************
    	// GIVEN  : A pointer to a trial to add to the session.
    	// TASK   : Add the trial to the session.             
    	// RETURN : Nothing.
    	// NOTE   : This method should be protected since only the friend class 
    	//          session_config should be allowed to invoke it.
    	//********************************************************************	
		
		void clear_block ();

};

#endif
