/*************************************************************************
 FILE:				TRIAL.H     
 CREATION DATE:		5 May 1996
 LAST MODIFIED:		5 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file defines 'trial_data', the base class for trial data.  trial_data
 accommoates only the most basic trial data imaginable:  trial time, and
 whether the trial was an error.  .
*************************************************************************/  

#ifndef __T_DATA_H
#define __T_DATA_H

//#include "boolean.h"
#include <fstream.h>
#include <vector>
#include <time.h>

#define UNDEFINED 0;

/*
	Following constant used to convert from external representation of distances
	to internal values.  One mm is stored as this many units an an integer variable.
	Thus, if this value is 10, we store distances in units of tenths of a mm.
*/
#define DIST_UNIT 10.0F

/*
	A point in the movement trail.
	I originally made this an inner classof trial_data, but 
	I couldn't get the vector template class to work with it.
*/
class Sample
{
public:
	float x;	// in mm, from center of target
	float y;	// in mm, from center of target
	int time;	// in ms, from opening click
	typedef enum {BUTTON_UP = 0, CLICK_INSIDE_TARGET, CLICK_OUTSIDE_TARGET, SESSION_STOPPED}
			ButtonState;
	ButtonState button;
	
	Sample () : x (0.0F), y (0.0F), time (0), button(BUTTON_UP) {};
	Sample (float x, float y, int t, ButtonState butt) :
			x (x), y (y), time (t), button (butt) {};

	friend ostream& operator << (ostream& out, const Sample& s);
	
	// The following are required for any class which is passed to vector<>
	bool operator < (const Sample& s) const { return time < s.time;};
	bool operator == (const Sample& s) const {return x == s.x &&
		y == s.y && time == s.time && button == s.button;};
};

class session;

class trial_data : public CObject
{
	public:
		// DATA
		int target_diameter; // Diameter of target circle, in mm * DIST_UNIT
		int target_distance; // Distance of target circle from start, in mm * DIST_UNIT
		int target_angle;    // Angle of target circle w.r.t. home, in degrees
		float x_target_off, y_target_off;	// Distance of click from target center, in mm
//		int target_x, target_y; // Screen coords of target circle
//		int home_x, home_y;	// Screen coords of home square      
		int trial_time; // How long did trial take?
		bool tooLarge;	// Was the trial too large to fit on this screen?
		bool error;  // Was the trial an error trial?
		session* sess;	// The session which this trial is a part of
		time_t start_time;	// Time of day that participant began trial
		int trial_seq_num;	// The number of this trial within the session (1 == first)

		// Data structure for movement trial
		typedef std::vector<Sample> SampleVec;
		SampleVec trail;
		int nextSample;		// Next place to store a movement sample
		enum {TRAIL_START_SIZE = 3000};
		
		// METHODS
		trial_data(int diam, int dist, int ang, int trailSize = 0);	  
		// GIVEN : A diameter, distance, and angle with which to initialize a new
		//		   instance of trial_data 
		// TASK  : Class constructor:  initialize fresh instance of trial_data.
		// RETURN: A fresh instance of trial_data	
		
		trial_data () {target_diameter = 0; target_distance = 0; target_angle = 0; sess = NULL; nextSample = 0;};
		// GIVEN : Nothing.
		// TASK  : Create an instance with default values.
		// RETURN: A fresh instance of trial_data.


		trial_data (const trial_data& copy);
		// Copy constructor

		virtual trial_data* new_instance(int diam, int dist, int ang, int trailSize = 0);
		// GIVEN : A diameter, distance, and angle with which to initialize a new
		//		   instance of trial_data 
		// TASK  : Create a heap-allocated instance of trial_data.
		// RETURN: A pointer to a fresh instance of trial_data	

		virtual trial_data* copy_instance ();
		// GIVEN:  This instance of trial_data;
		// TASK:   Creat a heap-allocated instance with identical values to this
		// RETURN: A pointer to the new instance.
		
		virtual void add_sample (float x, float y, int time,
			Sample::ButtonState button = Sample::BUTTON_UP);
		// GIVEN : The location, time, and left button state of a sample
		// TASK  : Add this sample to the trail
		// RETURN: Nothing.	

		virtual bool is_trail ();
		// GIVEN : Nothing.
		// TASK  : Test for presence of a trails file.
		// RETURN: True if we are recording the trail for this trial.

		virtual void write_trail (ostream& out);
		// GIVEN : The file to write trails to.
		// TASK  : Write out the trail for this trial, if any.
		// RETURN: Nothing.	

		virtual void set_session (session* s) {sess = s;};

		virtual void set_trial_seq_num (int seq) {trial_seq_num = seq;};

		friend ofstream& operator << (ofstream& of, trial_data &td);
		// GIVEN : An ofstream to write to, and a trial_data instance. 
		// TASK  : Write appropriate data fields of trial_data to ofstream. 
		// RETURN: Nothing.
	}; 
	
#endif
