/*************************************************************************
 FILE:				TBD.H     
 CREATION DATE:		10 May 1996
 LAST MODIFIED:		10 May 1996
 AUTHOR:			Chris Hundhausen 
 COPYRIGHT:			(c) 1996 by 
 					Interactive Systems Group
 					Department of Computer and Information Science
 			 		University of Oregon
 			 		Eugene, OR  97403-1202
 			 		All rights reserved.	
 =========================================================================
 DESCRIPTION:
 This file defines the task-block definition (TBD) class, which is the 
 principal conceptual entity in WAPWAD's conceptual model.   A TBD 
 encapsulates three definitions:  (1) task, which includes general
 input device action (in the current implementation, only pointing and 
 dragging are supported), and a method for handling error trials;  (2) 
 block, which consists of a range of target diameters, target
 distances, and target angles to be crossed; and (3) data file format,
 which indicates which data should be saved to data files.
*************************************************************************/  

#ifndef __TBD_H
#define __TBD_H

#include "trial.h"
#include "list.h"
#include "session.h"

class task_block_def {
	public:
		/* DATA */
		//We support two types of tasks:  discrete pointing and discrete dragging.
		enum task_type {pointing, dragging}; 
		// We support three error handling techniques:
		// redo_at_random:  If trial is not performed successfully, simply requeue the
		//					diam/distance/angle combination, so that it is re-presented
		//					to the user at random.
		// redo-at_end:		If trial is not performed successfully, place it in an
		//					'error trial' queue; trials in the 'error trial' queue are
		//					the presented to the user after all trials have been presented.
		// dont_redo:		If trial is not performed successfully, do not force user to redo it.
		enum error_handling_technique {redo_at_random, redo_at_end, dont_redo};
		// We support two forms in which the duration of a practice session may
		// be specified: in terms of trials and minutes.
		enum practice_session_units {trials, minutes};
		// The exp_data struct specifies which experimental data should be saved for each block.
		// It allows the user to customize the contents of the data file.
		struct exp_data {
			boolean trial_time;
			boolean movement_microstruct;
			boolean home_square_loc;
			boolean target_loc;
			boolean cursor_ending_loc;
			boolean target_diameter;
			boolean target_distance;
			boolean target_angle;
			boolean error; 
			};
		list target_diameters; // Diameter of target circles, in mm
		list target_distances; // Distances of target circle from start, in mm
		list target_angles;    // Angles of target circle w.r.t. home, in degrees
		int trials_per_combination // Number of trials of each diam/distance/angle combination
		int practice_session_len; // The length of the practice session, in either trials or minutes
		practice_session_units ps_units; // Units in which practice session, if any, is specified
		// task_type t_type;		  // Task type.
		error_handling_techniques err_tech;	// Error handling technique
		
		// exp_data data_to_save; // The data to be saved in the data file.

		/* METHODS */
		
		task_block_def(boolean set_defaults);
		//*************************************************************************
    	// GIVEN  : A boolean 'set_defaults'.
    	// TASK   : Task_block_def constructor:  Initialize a new instance 
    	//			of task_block_def. If set_defaults is TRUE, then initialize
    	// 			with reasonable default values.  If set_defaults is FALSE, then
		//			initialize a completely blank instance; don't bother to assign
		//			default values.
    	// RETURN : A fresh instance of task_block_def.
    	//*************************************************************************

		friend char* operator >> (ifstream& infile, task_block_def &tbd);
		//*************************************************************************
    	// GIVEN  : References to an input file infile and a _blank_ instance of 
    	//			task_block_def tbd (that is, an instance of task_block_def 
		//			obtained via a call to the constructor with set_defaults = FALSE).
    	// TASK   : If possible, initialize values of tbd based on contents of infile.
    	// RETURN : A string containing the result of the attempted initialization.
    	//			If all went well, "SUCCESS" is returned.  If there existed an
    	//			error in infile, an appropriate error message beginning with
    	//			"ERROR," and describing the error, is returned. 
    	//*************************************************************************

		*session generate_participant_session();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Generate a single participant session (one block) based on the 
		//			configuration specified in self's data members.
    	// RETURN : A new session instance ready to be fed to the task environment.
    	//*************************************************************************
		
		*session generate_practice_session();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Generate a single participant practice session (bounded by 
    	//			specifed practice session length) based on the 
		//			configuration specified in self's data members.
    	// RETURN : A new session instance ready to be fed to the task environment.
    	//*************************************************************************
		

		*session generate_experimentor_session();
		//*************************************************************************
    	// GIVEN  : Nothing.
    	// TASK   : Generate an experimentor session (open-ended) based on the 
		//			configuration specified in self's data members.
    	// RETURN : A new session instance ready to be fed to the task environment.
    	//*************************************************************************
		
	
	}; 

#endif
